@@ -33,6 +33,8 @@ class DataPrep:
 
         x_datapreped_path.write(x)
         y_datapreped_path.write(y)
+        self.data.x = x_datapreped_path
+        self.data.y = y_datapreped_path
 
     def dataprep_method(self, x: pd.DataFrame, y: pd.DataFrame) -> Tuple[pd.DataFrame, pd.DataFrame]:
         """To be implemented in daughter class.
