@@ -11,6 +11,7 @@ from transparentpath import TransparentPath
 import logging
 
 from .configs import NodePublicConfig, NodeDataConfig
+from .messenger import NodeMessenger
 from .fitters import DecisionTreeFitter
 from .updaters import AdaBoostUpdater
 from .datapreps import BinFeaturesDataPrep
@@ -32,6 +33,8 @@ class Node:
     path_public_configs: TransparentPath
         Path to the json file containing public configuration of the node. Needs to be kept in memory to potentially
         update the node's id once set by the central server.
+    messenger: NodeMessages
+        Interface for exchanging messages with central server. See `ifra.messenger.NodeMessages`
     dataprep: Union[None, Callable]
         The dataprep method to apply to the data before fitting. Optional, specified in the node's public configuration.
     datapreped: bool
@@ -69,6 +72,9 @@ class Node:
     """Possible string values and corresponding updaters object for *updater* attribute of
     `ifra.configs.NodePublicConfig`"""
 
+    timeout_central = 600
+    """How long the node is supposed to wait for the central server to start up"""
+
     def __init__(
         self,
         path_public_configs: Union[str, TransparentPath],
@@ -76,39 +82,58 @@ class Node:
     ):
 
         self.public_configs = NodePublicConfig(path_public_configs)
+        self.path_public_configs = self.public_configs.path  # Will be a TransparentPath
+
+        if self.public_configs.local_model_path.parent.is_dir():
+            if not len(list(self.public_configs.local_model_path.parent.ls(""))) == 0:
+                raise ValueError(
+                    "Path where Node model should be saved is not empty. Make sure you cleaned all previous learning"
+                    " output"
+                )
+        else:
+            self.public_configs.local_model_path.parent.mkdir()
+
+        self.path_messages = self.path_public_configs.parent / "messages.json"
+
+        first_message = True
+        t = time()
+        while not self.path_messages.is_file():
+            if first_message:
+                logger.info("Waiting for central server to start before creating node...")
+                first_message = False
+            sleep(1)
+            if time() - t > self.timeout_central:
+                raise TimeoutError(f"Central server did not start after {self.timeout_central} seconds.")
+        if first_message is False:
+            logger.info("...central server started. Resuming node creation.")
+
+        self.messenger = NodeMessenger(self.path_messages)
+        self.data = NodeDataConfig(path_data)
+        self.data.dataprep_kwargs = self.public_configs.dataprep_kwargs
 
         if self.public_configs.fitter not in self.possible_fitters:
-            s = f"Fitter '{self.public_configs.fitter}' is not known. Can be one of:"
-            s = "\n".join([s] + list(self.possible_fitters.keys()))
-            raise ValueError(s)
+            function = self.public_configs.fitter.split(".")[-1]
+            module = self.public_configs.fitter.replace(f".{function}", "")
+            self.fitter = getattr(
+                __import__(module, globals(), locals(), [function], 0), function
+            )(self.public_configs, self.data)
+        else:
+            self.fitter = self.possible_fitters[self.public_configs.fitter](self.public_configs, self.data)
 
         if self.public_configs.updater not in self.possible_updaters:
-            s = f"Updater '{self.public_configs.updater}' is not known. Can be one of:"
-            s = "\n".join([s] + list(self.possible_updaters.keys()))
-            raise ValueError(s)
+            function = self.public_configs.updater.split(".")[-1]
+            module = self.public_configs.updater.replace(f".{function}", "")
+            self.updater = getattr(__import__(module, globals(), locals(), [function], 0), function)(self.data)
+        else:
+            self.updater = self.possible_updaters[self.public_configs.updater](self.data)
 
         if self.public_configs.dataprep is not None:
             if self.public_configs.dataprep not in self.possible_datapreps:
-                s = f"Dataprep '{self.public_configs.dataprep}' is not known. Can be one of:"
-                s = "\n".join([s] + list(self.possible_datapreps.keys()))
-                raise ValueError(s)
-            # function = self.public_configs.dataprep.split(".")[-1]
-            # module = self.public_configs.dataprep.replace(f".{function}", "")
-            # # self.dataprep = __import__(module, globals(), locals(), [function], 0)
-
-        self.path_public_configs = self.public_configs.path  # Will be a TransparentPath
-
-        if not len(list(self.public_configs.local_model_path.parent.ls(""))) == 0:
-            raise ValueError(
-                "Path where Node model should be saved is not empty. Make sure you cleaned all previous learning output"
-            )
-
-        self.data = NodeDataConfig(path_data)
-        self.data.dataprep_kwargs = self.public_configs.dataprep_kwargs
-        self.fitter = self.possible_fitters[self.public_configs.fitter](self.public_configs, self.data)
-        self.updater = self.possible_updaters[self.public_configs.updater](self.data)
-        if self.public_configs.dataprep is not None:
-            self.dataprep = self.possible_datapreps[self.public_configs.dataprep](self.data)
+                function = self.public_configs.dataprep.split(".")[-1]
+                module = self.public_configs.dataprep.replace(f".{function}", "")
+                self.dataprep = getattr(__import__(module, globals(), locals(), [function], 0), function)(self.data)
+            else:
+                self.dataprep = self.possible_datapreps[self.public_configs.dataprep](self.data)
         else:
             self.dataprep = None
 
@@ -117,7 +142,7 @@ class Node:
         self.ruleset = None
         self.last_fetch = None
 
-    def fit(self) -> RuleSet:
+    def fit(self) -> None:
         """Plots the features and classes distribution in `ifra.node.Node`'s *data.x* and
         `ifra.node.Node`'s *data.y* parent directories if `ifra.configs.NodePublicConfig` *plot_data*
         is True.
@@ -130,44 +155,49 @@ class Node:
         *_copy_for_learning* to their names. Sets `ifra.node.Node` *copied* to True.
         Calls the the fitter corresponding to `ifra.node.Node` *public_configs.fitter* on the node's features and
         targets and save the resulting ruleset in `ifra.node.Node` *public_configs.local_model_path*.
-
-        Returns
-        -------
-        RuleSet
-            `node.Node` *ruleset*
         """
-        if self.public_configs.plot_data:
-            if not (self.data.x.parent / "plots").isdir():
-                (self.data.x.parent / "plots").mkdir()
-            self.plot_data_histogram(self.data.x.parent / "plots")
 
-        if self.dataprep is not None and not self.datapreped:
-            logger.info(f"Datapreping data of node {self.public_configs.id}...")
-            self.dataprep.dataprep()
+        self.messenger.fitting = True
+
+        try:
             if self.public_configs.plot_data:
-                if not (self.data.x.parent / "plots_datapreped").isdir():
-                    (self.data.x.parent / "plots_datapreped").mkdir()
-                self.plot_data_histogram(self.data.x.parent / "plots_datapreped")
-            self.datapreped = True
-            logger.info(f"...datapreping done for node {self.public_configs.id}")
+                if not (self.data.x.parent / "plots").is_dir():
+                    (self.data.x.parent / "plots").mkdir()
+                self.plot_data_histogram(self.data.x.parent / "plots")
 
-        if not self.copied:
-            # Copy x and y in a different file, for it will be changed after each iterations by the update from central
-            x_suffix = self.data.x.suffix
-            self.data.x.cp(self.data.x.with_suffix("").append("_copy_for_learning").with_suffix(x_suffix))
-            self.data.x = self.data.x.with_suffix("").append("_copy_for_learning").with_suffix(x_suffix)
-            y_suffix = self.data.y.suffix
-            self.data.y.cp(self.data.y.with_suffix("").append("_copy_for_learning").with_suffix(y_suffix))
-            self.data.y = self.data.y.with_suffix("").append("_copy_for_learning").with_suffix(y_suffix)
-            self.copied = True
+            if self.dataprep is not None and not self.datapreped:
+                logger.info(f"Datapreping data of node {self.public_configs.id}...")
+                self.dataprep.dataprep()
+                if self.public_configs.plot_data:
+                    if not (self.data.x.parent / "plots_datapreped").is_dir():
+                        (self.data.x.parent / "plots_datapreped").mkdir()
+                    self.plot_data_histogram(self.data.x.parent / "plots_datapreped")
+                self.datapreped = True
+                logger.info(f"...datapreping done for node {self.public_configs.id}")
 
-        logger.info(f"Fitting node {self.public_configs.id} using {self.public_configs.fitter}...")
-        self.ruleset = self.fitter.fit()
-        logger.info(f"Found {len(self.ruleset)} rules in node {self.public_configs.id}")
-        self.ruleset_to_file()
-        logger.info(f"... node {self.public_configs.id} fitted, results saved in"
+            if not self.copied:
+                # Copy x and y in a different file, for it will be changed after each iterations by the update from
+                # central
+                x_suffix = self.data.x.suffix
+                self.data.x.cp(self.data.x.with_suffix("").append("_copy_for_learning").with_suffix(x_suffix))
+                self.data.x = self.data.x.with_suffix("").append("_copy_for_learning").with_suffix(x_suffix)
+                y_suffix = self.data.y.suffix
+                self.data.y.cp(self.data.y.with_suffix("").append("_copy_for_learning").with_suffix(y_suffix))
+                self.data.y = self.data.y.with_suffix("").append("_copy_for_learning").with_suffix(y_suffix)
+                self.copied = True
+
+            logger.info(f"Fitting node {self.public_configs.id} using {self.public_configs.fitter}...")
+            self.ruleset = self.fitter.fit()
+            logger.info(f"Found {len(self.ruleset)} rules in node {self.public_configs.id}")
+            self.ruleset_to_file()
+
+            self.messenger.fitting = False
+            logger.info(f"... node {self.public_configs.id} fitted, results saved in"
                     f" {self.public_configs.local_model_path.parent}.")
-        return self.ruleset
+        except Exception as e:
+            self.messenger.fitting = False
+            # Do not set messenger's error, done in self.watch
+            raise e
 
     def update_from_central(self, ruleset: RuleSet) -> None:
         """Modifies the files pointed by `ifra.node.Node`'s *data.x* and `ifra.node.Node`'s *data.y* by
@@ -195,7 +225,7 @@ class Node:
         iteration = 0
         name = self.public_configs.local_model_path.stem
         path = self.public_configs.local_model_path.parent / f"{name}_{iteration}.csv"
-        while path.isfile():
+        while path.is_file():
             iteration += 1
             path = self.public_configs.local_model_path.parent / f"{name}_{iteration}.csv"
 
@@ -235,7 +265,7 @@ class Node:
             iteration = 0
             name = name.replace(' ', '_')
             path_x = path / f"{name}_{iteration}.pdf"
-            while path_x.isfile():
+            while path_x.is_file():
                 iteration += 1
                 path_x = path_x.parent / f"{name}_{iteration}.pdf"
             fig.savefig(path_x)
@@ -249,7 +279,7 @@ class Node:
 
         iteration = 0
         path_y = path / f"classes_{iteration}.pdf"
-        while path_y.isfile():
+        while path_y.is_file():
             iteration += 1
             path_y = path_y.parent / f"classes_{iteration}.pdf"
 
@@ -283,40 +313,38 @@ class Node:
                 self.new_data = True
 
             t = time()
-            started = False
             stop_message = True
             new_central_model = True  # start at true to trigger fit even if no central model is here at first iteration
+            logger.info(f"Starting node {self.public_configs.id}")
+            self.messenger.running = True
+            del_messenger = False
+
             while time() - t < timeout or timeout <= 0:
-                if not (self.path_public_configs.parent / "running").isfile():
-                    (self.path_public_configs.parent / "running").touch()
+
                 self.public_configs = NodePublicConfig(self.path_public_configs)
+                self.messenger.get_latest_messages()
                 if self.public_configs.id is None:
                     raise ValueError(
                         "Node id is not set, meaning central server is not running. Always start central server first."
                     )
-                if started is False:
-                    logger.info(f"Starting node {self.public_configs.id}")
-                    started = True
-                if self.public_configs.central_error is not None:
+                if self.messenger.central_error is not None:
                     logger.info(f"Central server crashed. Stopping node {self.public_configs.id}.")
+                    del_messenger = True
                     stop_message = False
-                    del self.public_configs.configs["central_error"]
-                    self.public_configs.save()
                     break
-                if self.public_configs.stop:
+                if self.messenger.stop:
                     logger.info(f"'stop' flag is True. Stopping learning in node {self.public_configs.id}")
+                    del_messenger = True
                     stop_message = False
-                    del self.public_configs.configs["stop"]
-                    self.public_configs.save()
                     break
 
                 if self.last_fetch is None:
-                    if self.public_configs.central_model_path.isfile():
+                    if self.public_configs.central_model_path.is_file():
                         get_ruleset()
                         new_central_model = True
                 else:
                     if (
-                        self.public_configs.central_model_path.isfile()
+                        self.public_configs.central_model_path.is_file()
                         and self.public_configs.central_model_path.info()["mtime"] > self.last_fetch.timestamp()
                     ):
                         get_ruleset()
@@ -331,9 +359,12 @@ class Node:
                 logger.info(f"Timeout of {timeout} seconds reached, stopping learning in node"
                             f" {self.public_configs.id}.")
 
-            (self.path_public_configs.parent / "running").rm(absent="ignore")
+            self.messenger.running = False
+            self.messenger.stop = True
+
+            if del_messenger:
+                self.messenger.rm()
         except Exception as e:
-            (self.path_public_configs.parent / "running").rm(absent="ignore")
-            self.public_configs.error = traceback.format_exc()
-            self.public_configs.save()
+            self.messenger.running = False
+            self.messenger.error = traceback.format_exc()
             raise e
